@@ -20,3 +20,13 @@ testModel <- es(Mcomp::M3$N1984$x, "ANA", h=18, silent=TRUE)
 test_that("ETS(ANA) simulated from estimated model", {
     expect_match(simulate(testModel,nsim=10,seed=5,obs=100)$model, "ANA");
 })
+
+testModel <- auto.ssarima(Mcomp::M3$N1234$x, h=18, silent=TRUE)
+test_that("ARIMA(0,1,3) with drift simulated from estimated model", {
+    expect_match(simulate(testModel,nsim=10,seed=5,obs=100)$model, "(0,1,3)");
+})
+
+test_that("ARIMA(0,1,1) with intermittent data", {
+    expect_match(sim.ssarima(nsim=10,obs=100,iprob=0.2)$model, "iARIMA");
+})
+
